#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Download, parse and prepare the latest annual Consumer Price Index (CPI) dataset.

Source: The Bureau of Labor Statistics at the U.S. Department of Labor.
Survey: CPI-All Urban Consumers (Current Series)
        All items in U.S. city average, all urban consumers, not seasonally adjusted
Identifer: CUUR0000SA0
"""
import os
import csv
import requests
from datetime import date


class Downloader(object):
    this_dir = os.path.dirname(__file__)

    def update(self):
        """
        Update the Consumer Price Index dataset that powers this library.
        """
        self.get_tsv()
        self.parse_tsv()

    def get_tsv(self):
        """
        Download the latest annual Consumer Price Index (CPI) dataset.
        """
        bls_url = "https://download.bls.gov/pub/time.series/cu/cu.data.1.AllItems"
        response = requests.get(bls_url)
        with open(os.path.join(self.this_dir, 'data.tsv'), 'w') as f:
            f.write(response.text)

    def parse_period(self, row):
        """
        Accepts a row from the raw BLS data. Returns a Python date object based on its period.
        """
        # If it is annual data, return it as Jan. 1 of that year.
        if row['period'] == 'M13':
            return date(row['year'], 1, 1)
        # If it is month data, return it as the first day of the month.
        else:
            return date(row['year'], int(row['period'].replace("M", "")), 1)

    def parse_tsv(self):
        """
        Parse the downloaded fixed-width file from the BLS and convert it into a CSV.
        """
        # Get the raw data
        input = os.path.join(self.this_dir, "data.tsv")
        reader = csv.DictReader(open(input, 'r'), delimiter="\t")

        # Figure out where we're going to store the clean data
        output = os.path.join(self.this_dir, "data.csv")
        writer = csv.DictWriter(
            open(output, 'w'),
            fieldnames=["series", "period", "year", "date", "value"]
        )
        writer.writeheader()

        # Loop through it
        for row in reader:
            # Clean it up
            d = dict(
                series=row['series_id'].strip(),
                period=row['period'].strip(),
                year=int(row['year'].strip()),
                value=float(row['value'].strip())
            )
            # Only keep the annual totals (M13) from the series we care about.
            if d['series'] == 'CUUR0000SA0':
                d['date'] = self.parse_period(d)
                writer.writerow(d)


if __name__ == '__main__':
    Downloader().update()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Open the Consumer Price Index (CPI) dataset.
"""
import os
import csv


class Data(object):
    this_dir = os.path.dirname(__file__)

    def __init__(self):
        self.cpi_file = self.get_cpi_file()

    def get_year_dict(self):
        """
        Returns a dictionary of the CPI-U adjustment value for each year available.
        """
        return dict(
            (int(r['year']), float(r['value'])) for r in self.cpi_file if r['period'] == 'M13'
        )

    def get_cpi_file(self):
        """
        Returns the CPI data as a csv.DictReader object.
        """
        # Open up the CSV from the BLS
        csv_path = os.path.join(self.this_dir, 'data.csv')
        csv_file = open(csv_path, "r")
        return csv.DictReader(csv_file)


data = Data()
cpi_by_year = data.get_year_dict()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Quickly adjust U.S. dollars for inflation using the Consumer Price Index (CPI)
"""
import warnings
from datetime import date
from .data import cpi_by_year
from .download import Downloader
from .errors import CPIDoesNotExist, StaleDataWarning


# Establish the range of data available
YEARS = cpi_by_year.keys()
EARLIEST_YEAR = min(YEARS)
LATEST_YEAR = max(YEARS)

# Figure out how out of date you are
DAYS_SINCE_LATEST = (date.today() - date(LATEST_YEAR, 1, 1)).days

# If it's more than two and a half years out of date, raise a warning.
if DAYS_SINCE_LATEST > (365*2.5):
    warnings.warn(StaleDataWarning())


def get(year):
    """
    Returns the CPI value for a given year.
    """
    try:
        return cpi_by_year[year]
    except KeyError:
        raise CPIDoesNotExist("CPI value not found for {}".format(year))


def inflate(value, year, to=LATEST_YEAR):
    """
    Returns a dollar value adjusted for inflation.

    You must submit the value, followed by the year its from.

    By default, the input is adjusted to the most recent year available from the CPI.

    If you'd like to adjust to a different year, submit it as an integer to the optional `to` keyword argument.
    """
    # If the two years match, just return the value unadjusted
    if year == to:
        return value

    # Otherwise, let's do the math.
    # The input value is multiplied by the CPI of the target year,
    # then divided into the CPI from the source year.
    return (value * get(to)) / float(get(year))


def update():
    """
    Updates the core Consumer Price Index dataset at the core of this library.

    Requires an Internet connection.
    """
    Downloader().update()
